@@ -3,19 +3,23 @@ import json
 import os
 from tempfile import mkstemp
 from typing import Union, Any, Optional, Tuple, Dict
-
 import requests
 import globus_sdk
+
+from globus_sdk import BaseClient
+from globus_sdk.utils import slash_join
 from globus_sdk.authorizers import GlobusAuthorizer
-from globus_sdk.base import BaseClient, slash_join
 from mdf_toolbox import login, logout
 from mdf_toolbox.globus_search.search_helper import SEARCH_LIMIT
 from funcx.sdk.client import FuncXClient
+from globus_sdk.scopes import AuthScopes, SearchScopes
 
 from dlhub_sdk.config import DLHUB_SERVICE_ADDRESS, CLIENT_ID
 from dlhub_sdk.utils.futures import DLHubFuture
 from dlhub_sdk.utils.schemas import validate_against_dlhub_schema
 from dlhub_sdk.utils.search import DLHubSearchHelper, get_method_details, filter_latest
+from dlhub_sdk.utils.validation import validate
+from dlhub_sdk.utils.funcx_login_manager import FuncXLoginManager
 
 # Directory for authentication tokens
 _token_dir = os.path.expanduser("~/.dlhub/credentials")
@@ -41,6 +45,9 @@ class DLHubClient(BaseClient):
         You must provide authorizers DLHub for all sub-services: Globus Search, FuncX, and OpenID.
     """
 
+    # service name is a class attribute in Globus SDK 3
+    service_name = "DLHub"
+
     def __init__(self, dlh_authorizer: Optional[GlobusAuthorizer] = None,
                  search_authorizer: Optional[GlobusAuthorizer] = None,
                  fx_authorizer: Optional[GlobusAuthorizer] = None,
@@ -87,9 +94,8 @@ class DLHubClient(BaseClient):
                 logger.warning('You have defined some of the authorizers but not all. DLHub is falling back to login. '
                                'You must provide authorizers for DLHub, Search, OpenID, FuncX.')
 
-            fx_scope = "https://auth.globus.org/scopes/facd7ccc-c5f4-42aa-916b-a0e270e2c2a9/all"
             auth_res = login(services=["search", "dlhub",
-                                       fx_scope, "openid"],
+                                       FuncXClient.FUNCX_SCOPE, "openid"],
                              app_name="DLHub_Client",
                              make_clients=False,
                              client_id=CLIENT_ID,
@@ -100,25 +106,32 @@ class DLHubClient(BaseClient):
 
             # Unpack the authorizers
             dlh_authorizer = auth_res["dlhub"]
-            fx_authorizer = auth_res[fx_scope]
+            fx_authorizer = auth_res[FuncXClient.FUNCX_SCOPE]
             openid_authorizer = auth_res['openid']
             search_authorizer = auth_res['search']
 
         # Define the subclients needed by the service
-        self._fx_client = FuncXClient(fx_authorizer=fx_authorizer,
-                                      search_authorizer=search_authorizer,
-                                      openid_authorizer=openid_authorizer,
-                                      no_local_server=kwargs.get("no_local_server", True),
-                                      no_browser=kwargs.get("no_browser", True))
+
+        auth_dict = {
+            FuncXClient.FUNCX_SCOPE: fx_authorizer,
+            AuthScopes.openid: openid_authorizer,
+            SearchScopes.all: search_authorizer,
+            'dlhub': dlh_authorizer,
+         }
+
+        login_manager = FuncXLoginManager(authorizers=auth_dict)
+        self._fx_client = FuncXClient(login_manager=login_manager)
+
         self._search_client = globus_sdk.SearchClient(authorizer=search_authorizer,
-                                                      http_timeout=5 * 60)
+                                                      transport_params={"http_timeout": http_timeout})
 
         # funcX endpoint to use
         self.fx_endpoint = '86a47061-f3d9-44f0-90dc-56ddc642c000'
         self.fx_cache = {}
-        super(DLHubClient, self).__init__("DLHub", environment='dlhub',
+
+        super(DLHubClient, self).__init__(environment='dlhub',
                                           authorizer=dlh_authorizer,
-                                          http_timeout=http_timeout,
+                                          transport_params={"http_timeout": http_timeout},
                                           base_url=DLHUB_SERVICE_ADDRESS,
                                           **kwargs)
 
@@ -230,8 +243,8 @@ class DLHubClient(BaseClient):
         return get_method_details(metadata, method)
 
     def run(self, name: str, inputs: Any, parameters: Optional[Dict[str, Any]] = None,
-            asynchronous: bool = False, debug: bool = False, async_wait: float = 5,
-            timeout: Optional[float] = None)\
+            asynchronous: bool = False, debug: bool = False, validate_input: bool = True,
+            async_wait: float = 5, timeout: Optional[float] = None)\
             -> Union[
                 DLHubFuture,
                 Tuple[Any, Dict[str, Any]],
@@ -245,6 +258,7 @@ class DLHubClient(BaseClient):
             parameters: Any optional parameters to pass to the function.
             asynchronous: Whether to return from the function immediately or wait for the execution to finish.
             debug: Whether to capture the standard out and error printed during execution
+            validate_input: whether to validate the provided input against the servable's published metadata
             async_wait: How many seconds to wait between checking async status
             timeout: How long to wait for a result to return. Only used for synchronous calls
         Returns:
@@ -269,12 +283,31 @@ class DLHubClient(BaseClient):
             'parameters': parameters,
             'debug': debug
         }
+
+        if validate_input:
+            self._validate_input(name, inputs)
+
         task_id = self._fx_client.run(payload, endpoint_id=self.fx_endpoint, function_id=funcx_id)
 
         # Return the result
         future = DLHubFuture(self, task_id, async_wait, debug)
         return future.result(timeout=timeout) if not asynchronous else future
 
+    def _validate_input(self, name: str, inputs: Any) -> None:
+        """Validate user inputted type against model metadata
+
+        Args:
+            name (string): The name of the servable in question
+            inputs: Data whose types are to be compared to the metadata expectations
+        Returns:
+            None
+        Raises:
+            ValueError: If any value in inputs is unexpected
+            TypeError: If any type in inputs is unexpected
+        """
+        res = self.search(f"dlhub.name: {name}", advanced=True, limit=1)
+        validate(inputs, res[0]["servable"]["methods"]["run"]["input"])
+
     def run_serial(self, servables, inputs, async_wait=5):
         """Invoke each servable in a serial pipeline.
         This function accepts a list of servables and will run each one,
@@ -343,15 +376,14 @@ class DLHubClient(BaseClient):
         try:
             model.get_zip_file(zip_filename)
 
-            # Get the authorization headers
-            headers = {}
-            self.authorizer.set_authorization_header(headers)
+            # Get the authorization header token (string for the headers dict)
+            header = self.authorizer.get_authorization_header()
 
             # Submit data to DLHub service
             with open(zip_filename, 'rb') as zf:
                 reply = requests.post(
                     slash_join(self.base_url, 'publish'),
-                    headers=headers,
+                    headers={"Authorization": header},
                     files={
                         'json': ('dlhub.json', json.dumps(metadata), 'application/json'),
                         'file': ('servable.zip', zf, 'application/octet-stream')
